"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jsdom_1 = require("jsdom");
class JsDomDocument {
    constructor(html) {
        this.syntaxHighlighterScriptClassName = "__syntaxHighlighterScript";
        this.dom = new jsdom_1.JSDOM(html, { runScripts: "dangerously" });
        this.document = this.dom.window.document;
    }
    addSyntaxHighlighterScript(contents) {
        this.addScript(contents, (se) => {
            se.className = this.syntaxHighlighterScriptClassName;
        });
    }
    addScript(contents, callback = function () { }) {
        const scriptEl = this.document.createElement("script");
        scriptEl.textContent = contents;
        callback(scriptEl);
        this.document.body.appendChild(scriptEl);
    }
    addToggleScript(contents) {
        this.addScript(contents);
    }
    addCss(css) {
        const style = this.document.createElement("style");
        style.innerHTML = css;
        this.document.head.appendChild(style);
    }
    getNewContents(isPartial) {
        if (isPartial) {
            return this.document.head.innerHTML + this.getBodyWithoutSyntaxHighlighterScripts();
        }
        else {
            return this.getDocTypeString() + this.getHtmlWithoutSyntaxHighlighterScripts();
        }
    }
    getDocTypeString() {
        var node = this.document.doctype;
        let html = "";
        if (node) {
            html = "<!DOCTYPE "
                + node.name
                + (node.publicId ? ' PUBLIC "' + node.publicId + '"' : '')
                + (!node.publicId && node.systemId ? ' SYSTEM' : '')
                + (node.systemId ? ' "' + node.systemId + '"' : '')
                + '>';
        }
        return html;
    }
    getHtmlWithoutSyntaxHighlighterScripts() {
        const htmlEl = this.document.documentElement;
        let html = htmlEl.outerHTML;
        return this.removeScripts(html);
    }
    getBodyWithoutSyntaxHighlighterScripts() {
        const body = this.document.body;
        let html = body.outerHTML;
        return this.removeScripts(html);
    }
    removeScripts(html) {
        const removableScripts = this.document.getElementsByClassName(this.syntaxHighlighterScriptClassName);
        for (let i = 0; i < removableScripts.length; i++) {
            html = html.replace(removableScripts[i].outerHTML, "");
        }
        return html;
    }
}
exports.JsDomDocument = JsDomDocument;
