"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class SyntaxHighlighterDocumentManager {
    constructor(jsDomDocument, minifier, assetLoader) {
        this.jsDomDocument = jsDomDocument;
        this.minifier = minifier;
        this.assetLoader = assetLoader;
    }
    addSyntaxHighlighterScripts(useMinifiedSyntaxHighlighter) {
        const scripts = this.assetLoader.getScripts(useMinifiedSyntaxHighlighter);
        scripts.forEach(s => this.jsDomDocument.addSyntaxHighlighterScript(s));
    }
    addCustomTheme(customTheme) {
        this.addMinifiableTheme(customTheme);
    }
    addAdditionalCss(additionalCss) {
        this.addMinifiableTheme(additionalCss);
    }
    addNamedTheme(themeName) {
        this.addTheme(this.assetLoader.getTheme(this.minifier.minifies, themeName));
    }
    addMinifiableTheme(theme) {
        this.addTheme(this.minifier.minifyCss(theme));
    }
    addTheme(theme) {
        this.jsDomDocument.addCss(theme);
    }
    getHighlightScript(globalParams, config) {
        return `
function merge(obj1, obj2)
{

    var result = {}, name;
    
    for (name in obj1)
        result[name] = obj1[name];

    for (name in obj2)
        result[name] = obj2[name];
    

    return result;
};
SyntaxHighlighter.config=merge(SyntaxHighlighter.config,${config})
SyntaxHighlighter.highlight(${globalParams});
`;
    }
    applySyntaxHighlighter(globalParams, config) {
        this.jsDomDocument.addSyntaxHighlighterScript(this.getHighlightScript(JSON.stringify(globalParams), JSON.stringify(config)));
    }
}
exports.SyntaxHighlighterDocumentManager = SyntaxHighlighterDocumentManager;
