"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const gulptransformbase_1 = require("gulptransformbase");
class SyntaxHighlighterTransform extends gulptransformbase_1.GulpTransformBase {
    constructor(options, assetLoader, minifier, jsDomDocumentFactory, syntaxHighlighterDocumentManagerFactory, toggleDocumentManagerFactory) {
        super(options);
        this.assetLoader = assetLoader;
        this.minifier = minifier;
        this.jsDomDocumentFactory = jsDomDocumentFactory;
        this.syntaxHighlighterDocumentManagerFactory = syntaxHighlighterDocumentManagerFactory;
        this.toggleDocumentManagerFactory = toggleDocumentManagerFactory;
        //#region defaults
        this.globalParams = {};
        this.config = {};
        this.themeName = "Default";
        this.useMinifiedSyntaxHighlighter = true;
        this.isPartial = function (html, file) {
            const firstTag = html.substring(html.indexOf("<"), html.indexOf(">")).toLowerCase();
            return !(firstTag.startsWith("<!doctype") || firstTag.startsWith("<html") || firstTag.startsWith("<head") || firstTag.startsWith("<body"));
        };
        //override defaults from options if provided
        if (options.useMinifiedSyntaxHighlighter !== undefined) {
            this.useMinifiedSyntaxHighlighter = options.useMinifiedSyntaxHighlighter;
        }
        if (options.isPartialFn) {
            this.isPartial = options.isPartialFn;
        }
        if (options.globalParams) {
            this.globalParams = options.globalParams;
        }
        if (options.theme) {
            this.themeName = options.theme;
        }
        if (options.config) {
            this.config = options.config;
        }
        this.initializeMinifier();
    }
    initializeMinifier() {
        var minifiedOutput = true;
        if (this.options.minifiedOutput !== undefined) {
            minifiedOutput = this.options.minifiedOutput;
        }
        this.minifier.initialize(minifiedOutput);
    }
    transformBufferFile(file, contents, encoding, cb) {
        this.file = file;
        this.html = contents.toString("utf8");
        this.jsDomDocument = this.jsDomDocumentFactory.create(this.html);
        this.setUpToggle();
        this.setUpSyntaxHighlighter();
        file.contents = new Buffer(this.getNewContents());
        cb(null, file);
    }
    setUpToggle() {
        if (this.options.toggleConfig) {
            var toggleDocumentManager = this.toggleDocumentManagerFactory.create(this.jsDomDocument, this.minifier);
            toggleDocumentManager.addToggle(this.options.toggleConfig);
        }
    }
    setUpSyntaxHighlighter() {
        var syntaxHighlighterDocumentManager = this.syntaxHighlighterDocumentManagerFactory.create(this.jsDomDocument, this.minifier, this.assetLoader);
        syntaxHighlighterDocumentManager.addSyntaxHighlighterScripts(this.useMinifiedSyntaxHighlighter);
        if (this.options.customTheme) {
            syntaxHighlighterDocumentManager.addCustomTheme(this.options.customTheme);
        }
        else {
            syntaxHighlighterDocumentManager.addNamedTheme(this.themeName);
        }
        if (this.options.additionalCss) {
            syntaxHighlighterDocumentManager.addAdditionalCss(this.options.additionalCss);
        }
        const toolbarFalse = { toolbar: false };
        syntaxHighlighterDocumentManager.applySyntaxHighlighter(Object.assign({}, this.globalParams, toolbarFalse), this.config);
    }
    getNewContents() {
        return this.jsDomDocument.getNewContents(this.isPartial(this.html, this.file));
    }
}
exports.SyntaxHighlighterTransform = SyntaxHighlighterTransform;
