"use strict";
///<reference path="./toggleDefns.d.ts"/>
///<reference path="./toggle.global.d.ts"/>
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
//attribute or config
function getToggleState(syntaxHighlighter, toggleConfig) {
    var dataToggle = syntaxHighlighter.getAttribute("data-toggleState");
    var toggleState = "Never";
    if (dataToggle === "Show") {
        toggleState = "Show";
    }
    else if (dataToggle === "Hide") {
        toggleState = "Hide";
    }
    else if (dataToggle === "Never") { //don't need this
        toggleState = "Never";
    }
    else {
        if (toggleConfig.toggleState === "Show") {
            toggleState = "Show";
        }
        else if (toggleConfig.toggleState === "Hide") {
            toggleState = "Hide";
        }
    }
    return toggleState;
}
function createToggle(highlighterElement, hide, messages, messageWhen, placement, classNames) {
    var isHidden = hide;
    var hasToggled = false;
    function createClickHandler(additionalOperations) {
        return function () {
            isHidden = !isHidden;
            if (additionalOperations) {
                additionalOperations();
            }
            performCommonOperations();
            hasToggled = true;
        };
    }
    function initializeToggle(toggle, clickHandler) {
        addClickHandler(toggle, clickHandler);
        addClassSVGSafe(toggle, classNames.toggle);
    }
    function initializeSingleToggle(toggle) {
        initializeToggle(toggle, createClickHandler());
    }
    function initializeBothToggles(toggleShow, toggleHide) {
        var bothToggleClicked = createClickHandler(addVisibleToggleToContainer);
        initializeToggle(toggleShow, bothToggleClicked);
        addClassSVGSafe(toggleShow, classNames.showToggle);
        initializeToggle(toggleHide, bothToggleClicked);
        addClassSVGSafe(toggleHide, classNames.hideToggle);
    }
    var isSingleToggle = false;
    var singleToggle;
    var singleDelay;
    var showToggle;
    var hideToggle;
    var singleOrBothToggles = createToggleElement();
    function determineSingleToggle(singleOrBoth) {
        if (singleOrBoth instanceof HTMLElement || singleOrBoth instanceof SVGElement) {
            singleToggle = singleOrBoth;
            return true;
        }
        var asSingle = singleOrBoth;
        if (asSingle.delay !== undefined) {
            singleDelay = asSingle.delay;
            singleToggle = asSingle.toggleElement;
            return true;
        }
        return false;
    }
    if (determineSingleToggle(singleOrBothToggles)) {
        isSingleToggle = true;
        initializeSingleToggle(singleToggle);
    }
    else {
        showToggle = singleOrBothToggles.showToggle;
        hideToggle = singleOrBothToggles.hideToggle;
        initializeBothToggles(showToggle, hideToggle);
    }
    var toggleContainer;
    var previousMessageElement;
    var previousVisibleToggle;
    function createToggleContainer() {
        toggleContainer = document.createElement("div");
        addClass(toggleContainer, classNames.toggleContainer);
        addClass(toggleContainer, classNames.toggleContainerInitial);
        setToggleContainerIsShowingClass();
    }
    function setToggleContainerIsShowingClass() {
        var classToRemove = classNames.toggleContainerHidden;
        var classToAdd = classNames.toggleContainerShown;
        if (isHidden) {
            classToRemove = classNames.toggleContainerShown;
            classToAdd = classNames.toggleContainerHidden;
        }
        addClass(toggleContainer, classToAdd);
        removeClass(toggleContainer, classToRemove);
    }
    function addVisibleToggleToContainer() {
        var visibleToggle = singleToggle;
        if (!isSingleToggle) {
            visibleToggle = isHidden ? showToggle : hideToggle;
        }
        if (previousVisibleToggle) {
            toggleContainer.replaceChild(visibleToggle, previousVisibleToggle);
        }
        else {
            toggleContainer.appendChild(visibleToggle);
        }
        previousVisibleToggle = visibleToggle;
    }
    function getMessage() {
        if (messageWhen === "Always" || messageWhen === "Hidden" && isHidden) {
            return isHidden ? messages.showMessage : messages.hiddenMessage;
        }
    }
    function createMessageElement(message) {
        var messageElement = document.createElement(placement === "Below" ? "div" : "span");
        messageElement.className = classNames.toggleText;
        messageElement.innerHTML = message;
        return messageElement;
    }
    function addMessageToToggleContainer() {
        var message = getMessage();
        if (message) {
            var messageElement = createMessageElement(message);
            if (previousMessageElement) {
                toggleContainer.replaceChild(messageElement, previousMessageElement);
            }
            else {
                toggleContainer.appendChild(messageElement);
            }
            previousMessageElement = messageElement;
        }
        else {
            if (previousMessageElement) {
                toggleContainer.removeChild(previousMessageElement);
            }
            previousMessageElement = undefined;
        }
    }
    function addToggleContainerBeforeHighlighter() {
        highlighterElement.parentNode.insertBefore(toggleContainer, highlighterElement);
    }
    function toggleHighlighter() {
        var collapsedClass = "collapsed";
        if (isHidden) {
            addClass(highlighterElement, collapsedClass);
        }
        else {
            removeClass(highlighterElement, collapsedClass);
        }
    }
    var singleDelayTimeout;
    function performCommonOperations() {
        if (!hasToggled) {
            removeClass(toggleContainer, classNames.toggleContainerInitial);
            addClass(toggleContainer, classNames.toggleContainerToggled);
        }
        setToggleContainerIsShowingClass();
        if (singleDelay) {
            if (singleDelayTimeout) {
                clearTimeout(singleDelayTimeout);
                singleDelayTimeout = undefined;
            }
            else {
                singleDelayTimeout = setTimeout(function () {
                    addMessageToToggleContainer();
                    toggleHighlighter();
                    singleDelayTimeout = undefined;
                }, singleDelay);
            }
        }
        else {
            addMessageToToggleContainer();
            toggleHighlighter();
        }
    }
    createToggleContainer();
    addVisibleToggleToContainer();
    addMessageToToggleContainer();
    addToggleContainerBeforeHighlighter();
    toggleHighlighter();
}
function getHideShowMessagesWhen(syntaxHighlighter, toggleConfigMessages) {
    function getHideShowMessages(message) {
        function getCaption() {
            var captionStr = "";
            var caption = syntaxHighlighter.children[0].caption;
            if (caption) {
                var text = caption.textContent;
                captionStr = text !== null ? text : "";
            }
            return captionStr;
        }
        var showMessage = "";
        var hideMessage = "";
        if (message.useTitle) {
            var caption = getCaption();
            showMessage = caption;
            hideMessage = caption;
        }
        else {
            showMessage = message.showMessage !== "" ? message.showMessage : message.message;
            hideMessage = message.hideMessage !== "" ? message.hideMessage : message.message;
        }
        showMessage = (message.prefixShow !== "" ? message.prefixShow : message.prefix) + showMessage;
        hideMessage = (message.prefixHide !== "" ? message.prefixHide : message.prefix) + hideMessage;
        return {
            hiddenMessage: hideMessage,
            showMessage: showMessage
        };
    }
    //attribute or config
    function mergeToggleConfigMessagesWithDataToggle() {
        var messageConfig = __assign({}, toggleConfigMessages);
        var messageNames = Object.getOwnPropertyNames(messageConfig);
        for (var i = 0; i < messageNames.length; i++) {
            var messageName = messageNames[i];
            var dataValue = syntaxHighlighter.getAttribute("data-toggle" + messageName);
            if (dataValue !== null) {
                messageConfig[messageName] = dataValue;
            }
        }
        return messageConfig;
    }
    var messages = mergeToggleConfigMessagesWithDataToggle();
    var hideShowMessages = getHideShowMessages(messages);
    return { when: messages.when, showMessage: hideShowMessages.showMessage, hiddenMessage: hideShowMessages.hiddenMessage };
}
function getSyntaxHighlighterElements() {
    return window.document.getElementsByClassName("syntaxhighlighter");
}
//#region helpers
function removeClass(target, className) {
    var replaced = target.className.replace(className, '');
    var splitReplaced = replaced.split(/\s+/);
    var newClassName = "";
    for (var i = 0; i < splitReplaced.length; i++) {
        newClassName += splitReplaced[i];
        if (i < splitReplaced.length - 1) {
            newClassName += " ";
        }
    }
    target.className = newClassName;
}
;
function addClass(target, className) {
    if (target.className == "") {
        target.className = className;
    }
    else {
        target.className = target.className + ' ' + className;
    }
}
;
function addClassSVGSafe(target, className) {
    var cls = target.getAttribute("class");
    if (cls) {
        cls += " " + className;
    }
    else {
        cls = className;
    }
    target.setAttribute("class", cls);
}
function addClickHandler(el, handler) {
    if (el.addEventListener) {
        el.addEventListener('click', handler, false);
    }
    else if (el.attachEvent) {
        el.attachEvent('onclick', handler);
    }
}
//#endregion
function setUpToggle(config, determineToggleState, determineMessages, addToggleFunctionality, syntaxHighlighterElementFinder) {
    determineToggleState = determineToggleState ? determineToggleState : getToggleState;
    determineMessages = determineMessages ? determineMessages : getHideShowMessagesWhen;
    addToggleFunctionality = addToggleFunctionality ? addToggleFunctionality : createToggle;
    syntaxHighlighterElementFinder = syntaxHighlighterElementFinder ? syntaxHighlighterElementFinder : getSyntaxHighlighterElements;
    var syntaxHighlighters = syntaxHighlighterElementFinder();
    for (var i = 0; i < syntaxHighlighters.length; i++) {
        var syntaxHighlighter = syntaxHighlighters[i];
        var toggleState = determineToggleState(syntaxHighlighter, config);
        if (toggleState !== "Never") {
            var messages = determineMessages(syntaxHighlighter, config.messages);
            var show = toggleState === "Show";
            addToggleFunctionality(syntaxHighlighter, !show, { hiddenMessage: messages.hiddenMessage, showMessage: messages.showMessage }, messages.when, config.messages.placement, config.classNames);
        }
    }
}
