"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
///<reference path="../../toggle/src/toggleDefns.d.ts"/>
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
class ToggleDocumentManager {
    constructor(jsDomDocument, minifier) {
        this.jsDomDocument = jsDomDocument;
        this.minifier = minifier;
        this.defaultToggleConfigMessages = {
            hideMessage: "",
            showMessage: "",
            message: "",
            prefix: "",
            prefixHide: "",
            prefixShow: "",
            useTitle: false,
            when: "Never",
            placement: "Right"
        };
        this.classNamesDefault = {
            toggleContainer: "toggleContainer",
            toggleText: "toggleText",
            toggle: "toggle",
            hideToggle: "hideToggle",
            showToggle: "showToggle",
            toggleContainerShown: "toggleContainerShown",
            toggleContainerHidden: "toggleContainerHidden",
            toggleContainerToggled: "toggleContainerToggled",
            toggleContainerInitial: "toggleContainerInitial"
        };
    }
    addToggle(toggleConfig) {
        this.addToggleDefaults(toggleConfig);
        this.addToggleJs();
        this.addToggleCss();
    }
    addToggleDefaults(toggleConfig) {
        var toggleConfigMessage = Object.assign({}, this.defaultToggleConfigMessages, toggleConfig.messages);
        var classNames = Object.assign({}, this.classNamesDefault, toggleConfig.classNames);
        this.css = toggleConfig.customCss ? toggleConfig.customCss : this.getDefaultToggleCss(classNames);
        this.createToggleFn = toggleConfig.createToggleFn ? toggleConfig.createToggleFn : this.defaultCreateToggleFn();
        this.toggleConfigWithDefaults = {
            classNames: classNames,
            messages: toggleConfigMessage,
            toggleState: toggleConfig.toggleState
        };
    }
    //#region toggle js
    addToggleJs() {
        this.jsDomDocument.addToggleScript(this.getToggleScript());
    }
    wrapInIEF(createToggleFn, toggleScript, setupToggle) {
        let ief = `
(function(){
    ${createToggleFn}
    ${toggleScript}
    ${setupToggle}
})()
`;
        return ief;
    }
    getJsonToggleConfigString() {
        return JSON.stringify(this.toggleConfigWithDefaults);
    }
    getSetUpToggleFunctionCall() {
        return `setUpToggle(${this.getJsonToggleConfigString()});`;
    }
    getToggleScript() {
        const script = this.wrapInIEF(this.createToggleFn, this.readToggle(), this.getSetUpToggleFunctionCall());
        return this.minifier.minifyScript(script);
    }
    readToggle() {
        return fs.readFileSync(path.join(__dirname, "toggle.js"), "utf8");
    }
    defaultCreateToggleFn() {
        return `
        function createToggleElement(){
            function createToggleShowHide(show){
                var xmlns="http://www.w3.org/2000/svg"
    
                var svg=document.createElementNS(xmlns,"svg");
                
                svg.setAttributeNS(null,"fill","none");
                svg.setAttributeNS(null,"stroke-linecap","round");
                svg.setAttributeNS(null,"stroke-linejoin","round");
                svg.setAttributeNS(null,"stroke-width","2");
                svg.setAttributeNS(null,"viewBox","0 0 24 24");
               
                var rect=document.createElementNS(xmlns,"rect");
                rect.setAttributeNS(null,"height",14);
                rect.setAttributeNS(null,"rx",7);
                rect.setAttributeNS(null,"ry",7);
                rect.setAttributeNS(null,"width",22);
                rect.setAttributeNS(null,"x",1);
                rect.setAttributeNS(null,"y",5);
    
                var circle=document.createElementNS(xmlns,"circle")
                circle.setAttributeNS(null,"cy",12);
                circle.setAttributeNS(null,"r",3);
                circle.setAttributeNS(null,"cx",show?16:8);
    
                svg.append(rect);
                svg.append(circle);
                
                return svg;
            }
            return {
                showToggle:createToggleShowHide(true),
                hideToggle:createToggleShowHide(false),
            }
        }
        `;
    }
    //#endregion
    //#region toggleCSS
    getDefaultToggleCss(classNames) {
        const svgSize = "1em";
        return `.${classNames.toggle}{ stroke:#000;top:0.125em;position:relative;height:${svgSize};width:${svgSize} }
        .${classNames.toggleText}{padding:10px; font-size:${svgSize}}
        `;
    }
    addToggleCss() {
        this.jsDomDocument.addCss(this.minifier.minifyCss(this.css));
    }
}
exports.ToggleDocumentManager = ToggleDocumentManager;
